function WriteAudio (x, FName, Fs, DFormat, FType)
% Write data to an audio file
% FType: 'WAVE' or 'AU'
% DFormat: 'mu-law8' (AU files), 'unsigned8' (WAVE files), 'integer8' (AU files),
%          'integer16', 'integer24' (WAVE files), or 'float32' (WAVE files) 

% $Id: WriteAudio.m,v 1.3 2006/09/27 18:11:55 pkabal Exp $

if (size (x, 1) == 1)
  x = x(:);
end
[AFPar.Nsamp, AFPar.Nchan] = size (x);

if (nargin < 5)
  FType = 'WAVE';
end
AFPar.FType = FType;

if (nargin < 4)
  DFormat = 'integer16';
end
AFPar.DFormat = DFormat;

AFPar.Sfreq = Fs;
AFPar.Fname = FName;
switch AFPar.FType
  case 'WAVE'
    Write_WAVE (x, AFPar); 
  case 'AU'
    Write_AU (x, AFPar);
  otherwise
    error ('WriteAudio: Unsuported file format');
end

AFPar.Fname = Full_Name (AFPar.Fname);
PrintAudioFilePar (AFPar);

return

% --------------------
function Write_WAVE (x, AFPar)

switch AFPar.DFormat
  case 'unsigned8'
    NBits = 8;
  case 'integer16'
    NBits = 16;
  case 'integer24'
    NBits = 24;
  case 'float32'
    NBits = 32;
  otherwise
    error ('WriteAudio: WAVE file, unsupported data format');
end

wavwrite (x, AFPar.Sfreq, NBits, AFPar.Fname);

return

% --------------------
function Write_AU (x, AFPar)

switch AFPar.DFormat
  case 'mu-law8'
    NBits = 8;
    Method = 'mu';
  case 'integer8'
    NBits = 8;
    Method = 'linear';
  case 'integer16'
    NBits = 16;
    Method = 'linear';
  otherwise
    error ('WriteAudio: AU file, unsuported data format');
end

auwrite (x, AFPar.Sfreq, NBits, Method, AFpar.FName);

return

% --------------------
function PrintAudioFilePar (AFPar)
% Print audio file parameters

switch AFPar.FType
  case 'WAVE'
    FType = 'WAVE';
  case 'AU'
    FType = 'AU';
  case 'Sphere'
    FType = 'NIST Sphere';
  case 'AIFF'
    FType = 'AIFF';
  case 'Raw'
    FType = 'Headerless';
  otherwise
    FType = 'unknown';
end

switch AFPar.DFormat
  case 'integer8'
    DFormat = '8-bit integer';
  case 'unsigned8';
    DFormat = 'offset-binary 8-bit integer';
  case 'integer16'
    DFormat = '16-bit integer';
  case 'integer24'
    DFormat = '24-bit integer';
  case 'integer32'
    DFormat = '32-bit integer';
  case 'float32'
    DFormat = '32-bit float';
  case 'float64'
    DFormat = '64-bit float';
  case 'mu-law8'
    DFormat = '8-bit mu-law';
  case 'A-law8'
    DFormat = '8-bit A-law';
  otherwise
    DFormat = 'unknown';
end

fprintf (' %s file: %s\n', FType, AFPar.Fname);
fprintf ('   Number of samples : %d (%.4g s)\n', AFPar.Nsamp, ...
              AFPar.Nsamp / AFPar.Sfreq);
fprintf ('   Sampling frequency: %g\n', AFPar.Sfreq);
fprintf ('   Number of channels: %d (%s)\n', AFPar.Nchan, DFormat);

% --------------------
function Fname = Full_Name (filename)

% Copied from Misc/FullName. See that routine for details.

Fname = [];
if (exist (filename, 'file') == 2)

  [pathstr name ext versn] = fileparts (filename);
  bname = [name ext versn];

  % Try to find the file in the current directory or on the
  % Matlab path
  if (isempty (pathstr))
    Fname = which (bname);
    [pathstr name ext versn] = fileparts (Fname);
    bname = [name ext versn];
  end

  % Path is not empty, but could be relative
  if (~ isempty (pathstr))
    if (exist (pathstr, 'dir') == 7)
      dirn = pwd;
      cd (pathstr);
      Fname = which (bname);
      cd (dirn);
    end
  end
  
end

if (isempty (Fname))
  Fname = filename;
end

return
